//Libraries
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27,20,4);  //LCD i2c adress sometimes 0x3f or 0x27

//Inputs
#define battery_voltage_in A2

//Outputs
#define PWM_out 10
#define load_enable 2

//Variables
float bat_voltage = 0;
int pwm_value = 0;
unsigned int before_millis = 0;
unsigned int now_millis = 0;
#define LCD_refresh_rate 1000


void setup(){
  pinMode(battery_voltage_in,INPUT);        //Set pins as INPUTS
  
  pinMode(PWM_out,OUTPUT);                  //Set pins as OUTPUTS
  digitalWrite(PWM_out,LOW);                //Set PWM to LOW so MSOFET is off  
  TCCR1B = TCCR1B & B11111000 | B00000001;  //timer 1 PWM frequency of 31372.55 Hz
  Serial.begin(9600);

  lcd.init();                     //Init the LCD
  lcd.backlight();                //Activate backlight  

  before_millis = millis;         //Used for LCD refresh rate
}

void loop(){  
  bat_voltage =   get_battery_voltage(10);
  
  now_millis = millis();
  if(now_millis - before_millis > LCD_refresh_rate)
  {
    before_millis = now_millis;
    //lcd.clear();
    lcd.setCursor(0,0);               //Column 0 row 0
    lcd.print(bat_voltage,2);       //Soalr voltage 
    lcd.print("V");                   //Volts
  }
  
  ///////////////////////////////////////////////
  //CONSTANT VOTLAGE CONTROL//
  if(bat_voltage > 12.0){
    pwm_value--;
    pwm_value = constrain(pwm_value,0,254);
  }  
  else {
    pwm_value++;
    pwm_value = constrain(pwm_value,0,254);
  }        
  analogWrite(PWM_out,pwm_value);
  ///////////////////////////////////////////////
  ///////////////////////////////////////////////
  
}//End void loop


/////////////////////////FUNCTIONS/////////////////////////
///////////////////////////////////////////////////////////
float get_battery_voltage(int n_samples)
{
  float voltage = 0;
  for(int i=0; i < n_samples; i++)
  {
    voltage += (analogRead(battery_voltage_in) * (5.0 / 1023.0) * 7.85);  //Voltage divider: 1/7+1 
  } 
  voltage = voltage/n_samples;
  return(voltage);
}
